/*
	bdd.c
	
	Esta biblioteca es una sencilla base de datos que muestra un ejemplo
	de mtodos para realizar
		- Altas
		- Bajas
		- Modificaciones
		- Informes
		Se basa en una lista dinmica lineal de estructuras.
		Se desde una arquitectura basada en un distribuidor.
*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#include "tipos.h"
#include "bdd.h"

void listado(GlobRef g)
{
	int i;
	/*
		Este procedimiento de listado solo muestra
		los registros ocupados
	*/
	RegistroRef p = g->bd;
	for(i=0;i<g->numRegistros;i++, p++)
		{
			if (!p->disponible)
				imprimir(p);
		}
}
void rellenar(RegistroRef unRegistro)
{
	/*
		Otra forma de lectura ms segura.

		Nota.- Podra quedar informacin en el bfer, suponiendo
		que el usuario inserte ms informacin que la admisible.
		
	*/
	charRef c = NULL;
	
	c = pedir("Deme el nombre : ");
	strncpy(unRegistro->nombre, c,LONGITUD_NOMBRE-1);
	free(c);
	
	c = pedir("Deme la direccin : ");
	strncpy(unRegistro->direccion, c,LONGITUD_DIRECCION-1);
	free(c);
	
	c = pedir ("Deme el telfono  : ");
	unRegistro->telefono = (int)strtol(c,NULL, 10);
	free(c);
	
	c = pedir("Deme el email: ");
	strncpy(unRegistro->email, c, LONGITUD_EMAIL-1);
	free(c);
	
	unRegistro->disponible = 0;
}

void imprimir(RegistroRef p)
{
	printf("nombre = %s, direccin = %s, telfono = %d, email = %s\n",
					p->nombre,
					p->direccion,
					p->telefono,
					p->email);
}

void altas(GlobRef g)
{
	/*
		Bsqueda de un registro disponible
	*/
	RegistroRef esteRegistro = NULL;
	int i;
	RegistroRef p = g->bd;
	int INCREMENTO = 50;
	printf("\n\nAltas\n\n");
	for(i=0;i<g->numRegistros;i++,p++)
		{
			if ( p->disponible )
				{
					esteRegistro = p;
					break;
				}
		}
	/*
		Rellenar ese registro
	*/
	if(esteRegistro)
		rellenar(esteRegistro);
	else
		{
			printf("\n\nImposible dar alta - ampliando la base de datos...");
			p = g->bd;
			p = realloc(	p, // Bloque redimensionado
										(g->numRegistros + INCREMENTO)*sizeof(Registro) // Nuevo tamao
									);
			if(NULL == p)
				{
					printf(" ..imposible. No hay espacio suficiente.\n\n");
				}
			else
				{
					g->bd = p;
					// Ponemos a cero los nuevos registros
					for(i=g->numRegistros;i<g->numRegistros+INCREMENTO;i++)
						p[i].disponible = 1;
					g->numRegistros += INCREMENTO;
				}
			printf("...terminado.\nNuevo nmero de registros: %d\n\n",
							g->numRegistros);
		}
}

void bajas(GlobRef g)
{
	#define LONGITUD_BUFFER 128
	char buf[LONGITUD_BUFFER];
	long int buscado;
	int i, seEncontro = 0;
	
	printf("\n\nBajas\n\n");
	listado(g);
	printf("\nIndique el registro que desea descartar (telfono): ");
	fgets(buf, LONGITUD_BUFFER, stdin);
	buf[strlen(buf)-1] = '\0';
	buscado=(int)strtol(buf,NULL,10);
	RegistroRef p = g->bd;
	for(i=0;i<g->numRegistros;i++,p++)
		{
			if (buscado == p->telefono)
				{
					p->disponible = 1;
					seEncontro = 1;
					break;
				}
		}
		if (!seEncontro)
			printf("\n\nPerdn, ese telfono no existe en la base de datos\n\n");
}

void modificaciones(GlobRef g)
{
	#define LONGITUD_BUFFER 128
	char buf[LONGITUD_BUFFER];
	long int buscado;
	int i, seEncontro = 0;
	
	printf("\n\nModificaciones\n\n");
	listado(g);
	printf("Indique el registro deseado (telfono): ");
	fgets(buf, LONGITUD_BUFFER, stdin);
	buf[strlen(buf)-1] = '\0';
	buscado=(int)strtol(buf,NULL,10);
	RegistroRef p = g->bd;
	for(i=0;i<g->numRegistros;i++,p++)
		{
			if (buscado == p->telefono)
				{
					rellenar(p);
					seEncontro = 1;
					break;
				}
		}
		if (!seEncontro)
			printf("\n\nPerdn, ese telfono no existe en la base de datos\n\n");
}

void informes(GlobRef g)
{
	#define LONGITUD_BUFFER 128
	char buf[LONGITUD_BUFFER];
	char buscado[LONGITUD_NOMBRE];
	int i, seEncontro = 0;
	
	printf("\n\nInformes\n\n");
	printf("\nIndique el nombre deseado: ");
	fgets(buf, LONGITUD_BUFFER, stdin);
	buf[strlen(buf)-1] = '\0';
	strncpy(buscado, buf,LONGITUD_NOMBRE-1);
	RegistroRef p = g->bd;
	for(i=0;i<g->numRegistros;i++,p++)
		{
			if (0 == strcmp(buscado,p->nombre))
				{
					imprimir(p);
					seEncontro = 1;
				}
		}
		if (!seEncontro)
			printf("\n\nPerdn, ese nombre no existe en la base de datos\n\n");
}

