#include "ficheros.h"

int contarCampos(  charRef cadena,
                   charRef separadores)
{
    int numTokens         = 0;
    charRef token         = NULL;
    int longitudCadena    = 0;
    charRef copia         = NULL;
    charRef dupCopia      = NULL; /* Copia de la direccin de cadena */
    /*
     Comprobacin de validez de los parmetros aportados.
     Estas comprobaciones requieren tiempo y se podran
     eliminar si garantizamos que el que hace la llamada
     cumpla su contrato y aporta valores vlidos.
     */
    if (!cadena)
    {
        printf("\n\nsplit - cadena nula\n\n");
        return -1;
    }
    if (!separadores)
    {
        printf("\n\nsplit - cadena de separadores nula\n\n");
        return -1;
    }

    longitudCadena = strlen(cadena);
    if (longitudCadena == 0)
    {
        return 0; /* El nmero de campos de una cadena vaca es 0 */
    }
    copia = malloc(longitudCadena+1);
    dupCopia = copia; /* Para reutilizar el bloque */
    strcpy(copia,cadena);
    do
    {
        token = strtok(copia, separadores);
        if (strlen(token)) numTokens++;
    } while(copia);
    free(dupCopia); // No leaks!
    return numTokens;
}

int split(  charRef cadena,
            charRef separadores,
            charRef ** fragmentos,
            intRef numFragmentos)
{
    charRef copia        = NULL;
    charRef dupCopia     = NULL;
    charRef token        = NULL;
    int numFrag          = 0;
    charRef * listaFrags = NULL;
    int numCampos        = contarCampos(cadena,separadores);
    if (0 == numCampos)
    {
        *fragmentos = NULL;
        *numFragmentos = 0;
        return 0;
    }
    listaFrags = malloc(numCampos*sizeof(charRef));
    copia = malloc(strlen(cadena) + 1);
    strcpy(copia,cadena);
    dupCopia = copia;
    do
    {
        token = strtok(copia, separadores);
        if (strlen(token))
        {
            listaFrags[numFrag] = malloc(strlen(token)+1);
            strcpy(listaFrags[numFrag], token);
            numFrag++;
        }
    } while(copia);
    free(dupCopia);
    *fragmentos = listaFrags;
    *numFragmentos = numCampos;
    return 1;
}

int leerLista(  charRef nombreDelFichero,
                charRef ** pListaDeLineas,
                intRef pNumeroDeLineasLeidas)
{
  charRef separador   = NULL;  /* Direccin de retorno o salto */
  charRef temp        = NULL;  /* Direccin de una lnea       */
  charRef linea       = NULL;  /* Linea temporal, optimizacin */
  size_t longitud     = 0;     /* longitud de esa lnea        */
  int n               = 0;     /* Contador de lneas           */
  int capacidad       = 500;   /* Capacidad inicial            */
  FILE * fp = fopen(nombreDelFichero, "r");
  if (!fp)
    {
        printf("\n\nError en leerLista: \"%s\" - %s\n\n\a",
        					nombreDelFichero,
        					strerror(errno));
        return 1;
    }
  /*
     Se reserva espacio para unas cuantas lneas
  */
  charRef * p = malloc(capacidad*sizeof(charRef));
  while (temp = fgetln(fp,&longitud))
    {
      linea = malloc(longitud+1); /* Para aadir el '\0' */
      memcpy(linea, temp,longitud);
      linea[longitud] = '\0';
      if (separador = strchr(linea,'\n')) *separador = '\0';
      if (separador = strchr(linea,'\r')) *separador = '\0';
      p[n] = linea;
      n++;
      if (n == capacidad)
      {
          capacidad +=500;
          p = realloc(p,capacidad*sizeof(charRef));
      }
    }
  fclose(fp);
  /*
  	La informacin se devuelve a travs de los argumentos
  */
  *pListaDeLineas        = p;
  *pNumeroDeLineasLeidas = n;
  return 0;
}

int leerTablaReales(			floatRef * p,
													int numFilas,
													int numCols,
													charRef nombreDelArchivo)
{
	int numDatos        = 0;     /* numFilas * numCols           */
	int i               = 0;     /* ndice de uso general        */
	int numErrores      = 0;		 /* Nmero de errores de lectura */
	floatRef temp       = NULL;  /* Bloque para almacenar datos  */
	FILE * fp           = NULL;

	numDatos            = numFilas * numCols;
	temp                = malloc(sizeof(float)*numDatos);

	fp                  = fopen(nombreDelArchivo, "r");
	if (fp)
    {
        for(i=0; i<numDatos;i++)
					if (1 != fscanf(fp,"%f",temp+i))
						{
							*(temp+i) = nanf(0);
							fscanf(fp, "%*s");
							numErrores++;
						}
        fclose(fp);
        *p = temp;
    }
	else
		printf("\nleerTablasReales - Error : %s\n", strerror(errno));
	return numErrores;
}


int importarTablaReales(    floatRef * p,
                            intRef pNumFilas,
                            intRef pNumCols,
                            charRef nombreDelArchivo,
                            charRef separadores)
{
    int i = 0, j = 0, k = 0;
    charRef * lista            = NULL;
    int numFilas               = 0;
    int numCols                = 0; /* Que habr que calcular */
    int numCampos              = 0;
    int numeroMaximoDeCampos   = 0;
    int numErrores             = 0;
    int error                  = 0;
    floatRef temp              = NULL;
    charRef * campos           = NULL;
    int noVacia                = 0;
    charRef proximo            = NULL;
    /* Lectura de informacin de disco */
    error = leerLista(nombreDelArchivo,&lista, &numFilas);
    if (error)
    {
        printf("\n\nError al importar tabla de nmeros reales\n\n");
        return -1;
    }
    else
    {
        /* Anlisis de informacin de disco */
        numeroMaximoDeCampos = contarCampos(lista[0],separadores);

        for(i=0;i<numFilas;i++)
        {
            numCampos = contarCampos(lista[i],separadores);
            if (numCampos > numeroMaximoDeCampos)
                numeroMaximoDeCampos = numCampos;
        }
        numCols = numeroMaximoDeCampos;
      /*
         Se crea un bloque de memoria con capacidad suficiente
         para almacenar todos los nmeros reales del archivo,
         suponiendo que todas las lneas contienen el mismo nmero
         de campos.
         */
      temp = malloc(sizeof(float)*numCols*numFilas);
      for(i=0;i<numFilas;i++)
        {
            noVacia = split(	lista[i],
            									separadores,
            									&campos,
            									&numCampos);
            if(noVacia)
            {
                for(k=0;k<numCampos;k++)
                {
                    temp[j] = strtof(campos[k],&proximo);
                    if (proximo == campos[k])
                    {
                        temp[j] = nan(0);
                        numErrores++;
                    }
                    j++;
                }
                /*
                 Si faltan campos, se aaden NaN al final
                 */
                if (numCampos < numCols)
                {
                    for(k=0;k<numCols-numCampos;k++)
                    {
                        temp[j] = nan(0);
                        numErrores++;
                        j++;
                    }
                }
            }
        }// fin del procesamiento
      *p         = temp; /* Se proporciona la tabla como puntero */
      *pNumCols  = numCols;
      *pNumFilas = numFilas;
    }
    return numErrores;
}

int mostrarTablaReales(floatRef tabla, int numFilas, int numCols)
{
	int i;
	charRef fmt = "%10.4f"; /* Formato encolumnado razonable     */
	charRef fRet = "%10.4f\n"; /* Formato encolumnado razonable     */
	int n = numFilas * numCols;
	if (tabla)
    {
        for(i=0;i<n;i++)
            printf((i+1)%numCols ? fmt:fRet,tabla[i]);
    }
	else
    {
        return 1;
    }
    return 0;
}
